#requires -version 2

$CStitle="Agent Health Check v1.2.113" # seagull/june 2019

#check to make sure this isn't being run as a component
if ($env:cs_profile_name) {
    write-host "This interactive script does not support being run as a component."
    write-host "Please run the script in the manner it was received."
    exit
}

#give it a spiffy title
$host.ui.RawUI.WindowTitle = "Datto RMM " + $CStitle

#paint the screen, black
$Host.UI.RawUI.BackgroundColor = 'Black'

if ($psversiontable.PSversion.build -ne 17763) {
    #do not manipulate the window size in 1809 because MS broke powershell for that build
    [console]::WindowWidth=88
    [console]::BufferWidth=[console]::WindowWidth
    [console]::WindowHeight=26
}

clear

##################################SCRIPT-WIDE VARIABLES#####################################

[System.Net.ServicePointManager]::ServerCertificateValidationCallback = {$true}
$ErrorActionPreference = "silentlycontinue"
if (([IntPtr]::size) -eq 4) {
    $CSProgramFiles='C:\Program Files'
    $CSRegSoftware='HKLM:\Software'
} else {
    $CSProgramFiles='C:\Program Files (x86)'
    $CSRegSoftware='HKLM:\Software\WOW6432Node'
}

$CSScriptDir = split-path -parent $MyInvocation.MyCommand.Definition

$CSAdmin=([bool](([System.Security.Principal.WindowsIdentity]::GetCurrent()).groups -match "S-1-5-32-544"))

[int]$CSKernel3 = ([System.Diagnostics.FileVersionInfo]::GetVersionInfo("C:\Windows\system32\kernel32.dll")).FileBuildPart # new for 2019
[int]$CSKernel4=(([System.Diagnostics.FileVersionInfo]::GetVersionInfo("C:\Windows\system32\kernel32.dll").FileVersion).split(".")[3] -replace '[^\d]','')

############################FUNCTIONS FOR BACKGROUND TASKS##################################

Function EpochTime {[int][double]::Parse((Get-Date -UFormat %s))}

function makeFolder {
    $script:CSResultDirectory = "$CSScriptDir\AHC-$env:computername-Result-$(EpochTime)"
    new-item "$CSResultDirectory\" -type directory -Force | out-null
}

function Get-HTTPHeaders { # by geoff varosky/sharepointyankee.com
    param( 
        [Parameter(ValueFromPipeline=$true)] 
        [string] $Url 
    )

    $request = [System.Net.WebRequest]::Create( $Url ) 
    $headers = $request.GetResponse().Headers 
    $headers.AllKeys | 
         Select-Object @{ Name = "Key"; Expression = { $_ }}, 
         @{ Name = "Value"; Expression = { $headers.GetValues( $_ ) } }
}

###########################FUNCTIONS TO PERFORM SCRIPT JOBS#################################

Function ProxyConfigMenu {
    write-host "This will configure the script to use Internet Explorer`'s"
    $choiceInput = Read-Host "proxy settings. To continue, push ENTER, or push Q to go back"
    switch ($choiceInput) {
    q {
        ChoosePlatform
        return
    }
    default {
            $proxyAddr = (get-itemproperty 'HKCU:\Software\Microsoft\Windows\CurrentVersion\Internet Settings').ProxyServer
            $proxy = new-object System.Net.WebProxy
            $proxy.Address = $proxyAddr
            $proxy.useDefaultCredentials = $true
            netsh winhttp show proxy
    }
    }
    $choiceInput = Read-Host "Push Q if your proxy server requires credentials or ENTER to proceed"
    switch ($choiceInput) {
    q {
        $webclient=New-Object System.Net.WebClient
        $SGLcreds=Get-Credential $null
        $webclient.Proxy.Credentials=$SGLcreds
        write-host "Proxy settings set:"
        Write-Host "   USERNAME: " -NoNewline -ForegroundColor Gray; Write-Host $SGLcreds.UserName -ForegroundColor White

        if ($proxy.Address.Length -le 1) {
            Write-Host "    ADDRESS: " -NoNewline -ForegroundColor Gray; Write-Host "Not set - Please confirm you have proxy settings configured in IE." -ForegroundColor Red
                $choiceInput = Read-Host "Script will now exit. Consider re-running script with no proxy settings configured"
                switch ($choiceInput) {
                default {
                exit
                }
            }
        } else {
            Write-Host "    ADDRESS: " -NoNewline -ForegroundColor Gray; Write-Host $proxy.Address -ForegroundColor White
        }
        $choiceInput = Read-Host "Press ENTER to proceed with the test"
        switch ($choiceInput) {
            default {
            ChoosePlatform
            return
            }
        }
    }
        default {
            ChoosePlatform
            return
        }
    }
}

Function ChoosePlatformBlue {
    write-host "Checking connectivity for Panda Cloud System Protection"
    write-host "-----------------------------"
    $script:CSWSAddress = 'https://01pandaws.centrastage.net/cs/version'
    $script:CSUpd8Address = 'https://sm-update.pandasecurity.com/cagupdate/Update.dll'
    $script:CSCSMAddress = 'https://sm.pandasecurity.com'
    $script:CSCCAddress = "01pandacc.centrastage.net"
    $script:CSTSAddress = "ts-eu-central.centrastage.net"
    $script:CAGVerAddress = "https://sm-update.pandasecurity.com/cagupdate/UpdateState.xml"
    $script:CSPlatIPs = @("8.8.8.8")
    $script:CSTSIPs = @("8.8.8.8")
}

Function ChoosePlatform {
    $script:CheckAgain = $true
    write-host "`r"
    write-host "Please select your Datto RMM Platform: " -ForegroundColor Cyan -NoNewline
    $script:CSServerChoice = Read-Host
    write-host "`r"
    switch ($script:CSServerChoice) {
        {[string]$script:CSServerChoice -eq 0} {
            if (!$CSAutoDetect) {
                write-host "Could not auto-detect (No Agent)." -ForegroundColor Red
            } else {
                &$CSAutoDetect
            }
        }
        {[int]$script:CSServerChoice -eq 1} {pinotage}
        {[int]$script:CSServerChoice -eq 2} {merlot}
        {[int]$script:CSServerChoice -eq 3} {syrah}
        {[int]$script:CSServerChoice -eq 4} {zinfandel}
        {[int]$script:CSServerChoice -eq 5} {concord}
        {[int]$script:CSServerChoice -eq 6} {ProxyConfigMenu}
        {[int]$script:CSServerChoice -eq 7} {panda}
    }
    
}

function pinotage {
    if (!$script:CSServerChoice) {break}
    $script:CSPlatform="Pinotage (Europe, Middle-East & Africa 1)"
    #web portal
    $script:CSCSMAddress = 'https://pinotage.centrastage.net'
    #monitoring service
    $script:CSMonitoringAddress = 'https://pinotage-monitoring.centrastage.net/device/1234/monitor'
    #control channel
    $script:CSCCAddress = "01cc.centrastage.net"
    #audit service
    $script:CSAuditAddress = 'https://pinotage-audit.centrastage.net/cs/version'
    #tunnel server
    $script:CSTSAddress = "ts.centrastage.net"
    #realtime service
    $script:CSRTAddress = 'https://pinotage-realtime.centrastage.net/notifications/test'
    #component/CS
    $script:CSCPTAddress = 'https://cpt.centrastage.net'
    #agent service
    $script:CSAgentAddress = 'https://pinotage-agent.centrastage.net/cs/version'
    #component/AWS
    $script:CSCPTAZAddress = 'https://cpt.centrastage.net.s3.amazonaws.com'
    #connectivity to agent update file
    $script:CSUpd8Address = 'https://update.centrastage.net/cagupdate/Update.dll'
    #platform IPs
    $script:CSPlatIPs = @("54.154.110.190","54.171.166.217","54.154.123.55","54.77.69.163","52.17.242.187","54.154.76.59","52.16.205.224","54.154.110.191","54.77.247.84","52.17.157.63","54.77.122.85","54.77.108.71","52.16.135.243","52.16.45.168","52.16.199.216","52.212.124.100")
    #tunnel server IPs
    $script:CSTSIPs = @("54.72.228.106","54.77.25.238","54.76.210.230","54.77.1.79","54.77.29.239","54.72.134.204","52.17.144.57","54.93.70.139","54.93.63.109","52.28.143.32","52.28.142.189","52.29.179.182","108.128.200.66","52.49.60.117","52.214.223.19","63.35.143.99")
    #agent version
    $script:CAGVerAddress = "https://update.centrastage.net/cagupdate/UpdateState.xml"
}

function merlot {
    if (!$script:CSServerChoice) {break}
    $script:CSPlatform="Merlot (Europe, Middle-East & Africa 2)"
    #web portal
    $script:CSCSMAddress = 'https://merlot.centrastage.net'
    #monitoring service
    $script:CSMonitoringAddress = 'https://merlot-monitoring.centrastage.net/device/1234/monitor'
    #control channel
    $script:CSCCAddress = "02cc.centrastage.net"
    #audit service
    $script:CSAuditAddress = 'https://merlot-audit.centrastage.net/cs/version'
    #tunnel server
    $script:CSTSAddress = "ts.centrastage.net"
    #realtime service
    $script:CSRTAddress = 'https://merlot-realtime.centrastage.net/notifications/test'
    #component/CS
    $script:CSCPTAddress = 'https://cpt-merlot.centrastage.net'
    #agent service
    $script:CSAgentAddress = 'https://merlot-agent.centrastage.net/cs/version'
    #component/AWS
    $script:CSCPTAZAddress = 'https://cpt-merlot.centrastage.net.s3.amazonaws.com'
    #connectivity to agent update file
    $script:CSUpd8Address = 'https://update-merlot.centrastage.net/cagupdate/Update.dll'
    #agent version
    $script:CAGVerAddress = "https://update-merlot.centrastage.net/cagupdate/UpdateState.xml"
    #platform IPs
    $script:CSPlatIPs = @("54.194.218.239","54.194.70.100","54.194.54.213","54.194.25.164","54.194.74.127","54.194.50.12","54.194.17.228","54.194.100.128","54.194.217.74","54.194.59.222","54.194.245.26","54.194.242.0","34.250.95.109","34.252.248.172","52.213.142.143","54.194.153.16")
    #tunnel server IPs
    $script:CSTSIPs = @("54.72.228.106","54.77.25.238","54.76.210.230","54.77.1.79","54.77.29.239","54.72.134.204","52.17.144.57","54.93.70.139","54.93.63.109","52.28.143.32","52.28.142.189","52.29.179.182","108.128.200.66","52.49.60.117","52.214.223.19","63.35.143.99")
}

function syrah {
    if (!$script:CSServerChoice) {break}
    $script:CSPlatform="Syrah (Asia-Pacific)"
    #web portal
    $script:CSCSMAddress = 'https://syrah.centrastage.net'
    #monitoring service
    $script:CSMonitoringAddress = 'https://syrah-monitoring.centrastage.net/device/1234/monitor'
    #control channel
    $script:CSCCAddress = "syrahcc.centrastage.net"
    #audit service
    $script:CSAuditAddress = 'https://syrah-audit.centrastage.net/cs/version'
    #tunnel server
    $script:CSTSAddress = "ts.centrastage.net"
    #realtime service
    $script:CSRTAddress = 'https://syrah-realtime.centrastage.net/notifications/test'
    #component/CS
    $script:CSCPTAddress = 'https://cpt-syrah.centrastage.net'
    #agent service
    $script:CSAgentAddress = 'https://syrah-agent.centrastage.net/cs/version'
    #component/AWS
    $script:CSCPTAZAddress = 'https://cpt-syrah.centrastage.net.s3.amazonaws.com'
    #connectivity to agent update file
    $script:CSUpd8Address = 'https://update-syrah.centrastage.net/cagupdate/Update.dll'
    #agent version
    $script:CAGVerAddress = "https://update-syrah.centrastage.net/cagupdate/UpdateState.xml"
    #platform IPs
    $script:CSPlatIPs = @("54.206.24.91","54.206.73.160","54.79.4.211","54.79.25.73","54.206.20.221","54.206.10.157","54.252.198.125","54.206.12.124")
    #tunnel server IPs
    $script:CSTSIPs = @("54.79.75.129","54.79.97.42","54.169.119.179","54.169.73.64")
}

function zinfandel {
    if (!$script:CSServerChoice) {break}
    $script:CSPlatform="Zinfandel (United States West)"
    #web portal
    $script:CSCSMAddress = 'https://zinfandel.centrastage.net'
    #monitoring service
    $script:CSMonitoringAddress = 'https://zinfandel-monitoring.centrastage.net/device/1234/monitor'
    #control channel
    $script:CSCCAddress = "03cc.centrastage.net"
    #audit service
    $script:CSAuditAddress = 'https://zinfandel-audit.centrastage.net/cs/version'
    #tunnel server
    $script:CSTSAddress = "ts.centrastage.net"
    #realtime service
    $script:CSRTAddress = 'https://zinfandel-realtime.centrastage.net/notifications/test'
    #component/CS
    $script:CSCPTAddress = 'https://cpt-zinfandel.centrastage.net'
    #agent service
    $script:CSAgentAddress = 'https://zinfandel-agent.centrastage.net/cs/version'
    #component/AWS
    $script:CSCPTAZAddress = 'https://cpt-zinfandel.centrastage.net.s3.amazonaws.com'
    #connectivity to agent update file
    $script:CSUpd8Address = 'https://update-zinfandel.centrastage.net/cagupdate/Update.dll'
    #agent version
    $script:CAGVerAddress = "https://update-zinfandel.centrastage.net/cagupdate/UpdateState.xml"
    #platform IPs
    $script:CSPlatIPs = @("54.201.158.254","52.26.220.111","54.186.19.220","52.11.249.126","54.213.57.149","54.213.162.73","54.201.132.105","54.201.211.18","54.200.154.205","52.27.100.233","34.210.221.105")
    #tunnel server IPs
    $script:CSTSIPs = @("54.191.106.204","54.187.170.202","54.200.178.30","52.34.11.113","54.207.102.187","54.207.105.254","13.52.116.82","52.9.238.130","54.177.238.11","54.67.12.67")
}

function concord {
    if (!$script:CSServerChoice) {break}
    $script:CSPlatform="Concord (United States East)"
    #web portal
    $script:CSCSMAddress = 'https://concord.centrastage.net'
    #monitoring service
    $script:CSMonitoringAddress = 'https://concord-monitoring.centrastage.net/device/1234/monitor'
    #control channel
    $script:CSCCAddress = "concordcc.centrastage.net"
    #audit service
    $script:CSAuditAddress = 'https://concord-audit.centrastage.net/cs/version'
    #tunnel server
    $script:CSTSAddress = "ts.centrastage.net"
    #realtime service
    $script:CSRTAddress = 'https://concord-realtime.centrastage.net/notifications/test'
    #component/CS
    $script:CSCPTAddress = 'https://cpt-concord.centrastage.net'
    #agent service
    $script:CSAgentAddress = 'https://concord-agent.centrastage.net/cs/version'
    #component/AWS
    $script:CSCPTAZAddress = 'https://cpt-concord.centrastage.net.s3.amazonaws.com'
    #connectivity to agent update file
    $script:CSUpd8Address = 'https://update-concord.centrastage.net/cagupdate/Update.dll'
    #agent version
    $script:CAGVerAddress = "https://update-concord.centrastage.net/cagupdate/UpdateState.xml"
    #platform IPs
    $script:CSPlatIPs = @("54.88.212.141","54.173.70.131","54.172.198.183","52.6.151.191","54.164.96.143","52.7.54.43","54.165.240.214","52.5.251.79","52.4.97.130","54.88.94.23","54.165.85.82","52.7.200.96","34.226.13.192","52.23.104.21","52.71.59.169","34.195.169.6")
    #tunnel server IPs
    $script:CSTSIPs = @("54.172.136.43","54.173.173.38","52.20.64.236","52.5.82.167","54.207.102.187","54.207.105.254","54.156.209.219","34.227.128.69","3.211.80.125","35.170.148.86","52.22.230.203","54.164.228.61")
}

function panda {
    if (!$script:CSServerChoice) {break}
    $script:CSPlatform="Panda Cloud Systems Management"
    write-host "-----------------------------"
    $script:CSWSAddress = 'https://01pandaws.centrastage.net/cs/version'
    $script:CSUpd8Address = 'https://sm-update.pandasecurity.com/cagupdate/Update.dll'
    $script:CSCSMAddress = 'https://sm.pandasecurity.com'
    $script:CSCCAddress = "01pandacc.centrastage.net"
    $script:CSTSAddress = "ts-eu-central.centrastage.net"
    $script:CAGVerAddress = "https://sm-update.pandasecurity.com/cagupdate/UpdateState.xml"
    $script:CSPlatIPs = @("8.8.8.8")
    $script:CSTSIPs = @("8.8.8.8")

}

Function SiteCheck {
    #cosmetic open
    $script:CSToolReport+="==================================`r`n"
    $script:CSToolReport+="`r`n"
    $script:CSToolReport+="= Connectivity Scan results for $CSPlatform`:`r`n"

    #web portal
        write-host "     Web Portal                  " -nonewline
        $CSHTTP_Request = [System.Net.WebRequest]::Create($CSCSMAddress)
        $CSHTTP_Response = $CSHTTP_Request.GetResponse()
        $CSHTTP_Status = [int]$CSHTTP_Response.StatusCode
        If ($CSHTTP_Status -eq 200) { 
            Write-Host "   OK!" -ForegroundColor Green -NoNewline
            $script:CSToolReport+="`+ SUCCESS: Web Portal               $CSCSMAddress`r`n"
        } else {
            Write-host "Failed" -ForegroundColor Red -NoNewline
            $script:CSToolReport+="`- FAILURE: Web Portal               $CSCSMAddress`r`n"
        }
        Clear-Variable CSHTTP_*

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #monitoring service
        write-host "Monitoring Service          " -nonewline
        $CSHTTP_Request = [System.Net.WebRequest]::Create($CSMonitoringAddress)
        $CSHTTP_Response = $CSHTTP_Request.GetResponse()
        $CSHTTP_Status = [int]$CSHTTP_Response.StatusCode
        If ($CSHTTP_Status -eq 200) { 
            Write-Host "   OK!" -ForegroundColor Green
            $script:CSToolReport+="`+ SUCCESS: Monitoring Service       " + $CSMonitoringAddress.substring(0,$CSMonitoringAddress.length-20) + "`r`n"
        } else {
            Write-host "Failed" -ForegroundColor Red
            $script:CSToolReport+="`- FAILURE: Monitoring Service       $CSMonitoringAddress`r`n"
        }
        Clear-Variable CSHTTP_*

     #cc
        write-host "     Control Channel             " -NoNewline
        if ($CSCCAddress.Length -le 2) {$CSCCAddress = "-NULL-"}
        $CSHTTP_ServerCheck = (New-Object net.sockets.tcpclient); $CSHTTP_ServerCheck.connect("$CSCCAddress",443)
        If ($CSHTTP_ServerCheck.Connected) { 
            Write-Host "   OK!" -ForegroundColor Green -NoNewline
            $script:CSToolReport+="`+ SUCCESS: Control Channel          $CSCCAddress`r`n"
        } else {
            Write-host "Failed" -ForegroundColor Red -NoNewline
            $script:CSToolReport+="`- FAILURE: Control Channel          $CSCCAddress`r`n"
        }
        $CSHTTP_ServerCheck.close()
        Clear-Variable CSHTTP_*

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #audit service
        write-host "Audit Service               " -NoNewline
        $CSHTTP_Request = [System.Net.WebRequest]::Create($CSAuditAddress)
        $CSHTTP_Response = $CSHTTP_Request.GetResponse()
        $CSHTTP_Status = [int]$CSHTTP_Response.StatusCode
        If ($CSHTTP_Status -eq 200) { 
            Write-Host "   OK!" -ForegroundColor Green
            $script:CSToolReport+="`+ SUCCESS: Audit Service            $CSAuditAddress`r`n"
        } else {
            Write-host "Failed" -ForegroundColor Red
            $script:CSToolReport+="`- FAILURE: Audit Service            $CSAuditAddress`r`n"
        }
        Clear-Variable CSHTTP_*

    #ts
        write-host "     Local Tunnel Server         " -NoNewline
        if ($CSTSAddress.Length -le 2) {$CSTSAddress = "-NULL-"}
            $CSHTTP_ServerCheck = (New-Object net.sockets.tcpclient); $CSHTTP_ServerCheck.connect("$CSTSAddress",443)
            If ($CSHTTP_ServerCheck.Connected) { 
                Write-Host "   OK!" -ForegroundColor Green -NoNewline
                $script:CSToolReport+="`+ SUCCESS: Local Tunnel Server      $CSTSAddress (Regional connectivity determined automatically)`r`n"
            } else {
                Write-host "Failed" -ForegroundColor Red -NoNewline
                $script:CSToolReport+="`- FAILURE: Local Tunnel Server      $CSTSAddress (Regional connectivity determined automatically)`r`n"
            }
        $CSHTTP_ServerCheck.close()
        Clear-Variable CSHTTP_*

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #real-time
        write-host "Real-time Service           " -nonewline
        $CSHTTP_Request = [System.Net.WebRequest]::Create($CSRTAddress)
        $CSHTTP_Response = $CSHTTP_Request.GetResponse()
        $CSHTTP_Status = [int]$CSHTTP_Response.StatusCode
        If ($CSHTTP_Status -eq 200) { 
            Write-Host "   OK!" -ForegroundColor Green
            $script:CSToolReport+="`+ SUCCESS: Real-time Service        $CSRTAddress`r`n"
        } else {
            Write-host "Failed" -ForegroundColor Red
            $script:CSToolReport+="`- FAILURE: Real-time Service        $CSRTAddress`r`n"
        }
        Clear-Variable CSHTTP_*


    #cpt
        write-host "     Component/CS                " -NoNewline
        $CSHTTP_Request = [System.Net.WebRequest]::Create($CSCPTAddress)
        try { 
            $CSHTTP_Request.GetResponse() | out-null
        } catch [System.Exception] {
            If ($_.Exception.Response.StatusCode.Value__ -eq 403) { 
                Write-Host "   OK!" -ForegroundColor Green -nonewline
                $script:CSToolReport+="`+ SUCCESS: Component/AWS            $CSCPTAZAddress`r`n"
            } else {
                Write-host "Failed" -ForegroundColor Red -nonewline
                $script:CSToolReport+="`- FAILURE: Component/AWS            $CSCPTAZAddress`r`n"
            }
        }
        if (!$error) {
            Write-Host "Failed" -ForegroundColor Red -nonewline
        }
        Clear-Variable CSHTTP_*

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #AgentAddress
        write-host "Agent Service               " -nonewline
        $CSHTTP_Request = [System.Net.WebRequest]::Create($CSAgentAddress)
        $CSHTTP_Response = $CSHTTP_Request.GetResponse()
        $CSHTTP_Status = [int]$CSHTTP_Response.StatusCode
        If ($CSHTTP_Status -eq 200) { 
            Write-Host "   OK!" -ForegroundColor Green
            $script:CSToolReport+="`+ SUCCESS: Agent Service            $CSAgentAddress`r`n"
        } else {
            Write-host "Failed" -ForegroundColor Red
            $script:CSToolReport+="`- FAILURE: Agent Service            $CSAgentAddress`r`n"
        }
        Clear-Variable CSHTTP_*

    #cptaz
        write-host "     Component/AWS               " -NoNewline
        $CSHTTP_Request = [System.Net.WebRequest]::Create($CSCPTAZAddress)
        try { 
            $CSHTTP_Request.GetResponse() | out-null
        } catch [System.Exception] {
            If ($_.Exception.Response.StatusCode.Value__ -eq 403) { 
                Write-Host "   OK!" -ForegroundColor Green -nonewline
                $script:CSToolReport+="`+ SUCCESS: Component/AWS            $CSCPTAZAddress`r`n"
            } else {
                Write-host "Failed" -ForegroundColor Red -nonewline
                $script:CSToolReport+="`- FAILURE: Component/AWS            $CSCPTAZAddress`r`n"
            }
        }
        if (!$error) {
            Write-Host "Failed" -ForegroundColor Red -nonewline
        }
        Clear-Variable CSHTTP_*

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #update
        write-host "Update Location             " -nonewline
        $CSHTTP_Request = [System.Net.WebRequest]::Create($CSUpd8Address)
        $CSHTTP_Response = $CSHTTP_Request.GetResponse()
        $CSHTTP_Status = [int]$CSHTTP_Response.StatusCode
        If ($CSHTTP_Status -eq 200) { 
            Write-Host "   OK!" -ForegroundColor Green
            $script:CSToolReport+="`+ SUCCESS: Update Location          $CSUpd8Address`r`n"
        } else {
            Write-host "Failed" -ForegroundColor Red
            $script:CSToolReport+="`- FAILURE: Update Location          $CSUpd8Address`r`n"
        }
        Clear-Variable CSHTTP_*

    #cosmetic closeout of first report segment
        $script:CSToolReport+="==================================`r`n"
        $script:CSToolReport+="`r`n"
        $script:CSToolReport+="= Tunnel Server IP Scan results (regionally-influenced by Platform selection):`r`n"

    #ts array check
        [int]$CSHTTTP_SuccessCount = '0'
        [int]$CSHTTP_ArrayPos = '0'       
        foreach ($iteration in $CSTSIPs) {

            #configure zeroes, for formatting
            if ($CSHTTP_SuccessCount -lt 10) {
                [string]$CSHTTP_FirstZero = "0"
            } else {
                [string]$CSHTTP_FirstZero = ""
            }

            if ($CSTSIPs.Count -lt 10) {
                [string]$CSHTTP_SecondZero = "0"
            } else {
                [string]$CSHTTP_SecondZero = ""
            }

            #the IP check itself
            $CSHTTP_ArrayPos++
            $CSHTTP_ServerCheck = (New-Object net.sockets.tcpclient); $CSHTTP_ServerCheck.connect("$iteration",443)
            write-host "`r     Tunnel Server IPs          " -nonewline; write-host ('[' + $CSHTTP_FirstZero + $CSHTTP_SuccessCount + '/' + $CSHTTP_SecondZero + $CSTSIPs.count + ']') -NoNewline -foregroundcolor DarkGray

            #conditional code to write results to the log as they file in
            if ($CSHTTP_ServerCheck.Connected) {
                $script:CSToolReport+= "`+ SUCCESS: $iteration `r`n"
                $CSHTTP_SuccessCount++
            } else {
                $script:CSToolReport+= "`- FAILURE: $iteration `r`n"
            }

            #wait a little bit
            Start-Sleep -m 100

            #configure zeroes again, for formatting
            if ($CSHTTP_SuccessCount -lt 10) {
                [string]$CSHTTP_FirstZero = "0"
            } else {
                [string]$CSHTTP_FirstZero = ""
            }

            #finish up
            if ($CSHTTP_ArrayPos -eq $CSTSIPs.count) {
                write-host "`r     Tunnel Server IPs          " -NoNewline;
                if ($CSHTTP_SuccessCount -ne $CSTSIPs.count) {
                    write-host ('[' + $CSHTTP_FirstZero + $CSHTTP_SuccessCount + '/' + $CSHTTP_SecondZero + $CSTSIPs.count + ']') -NoNewline -foregroundcolor Red
                } else {
                    write-host ('[' + $CSHTTP_FirstZero + $CSHTTP_ArrayPos + '/' + $CSHTTP_SecondZero + $CSTSIPs.count + ']') -NoNewline -foregroundcolor Green
                }
            }
        $CSHTTP_ServerCheck.close()
        }

        Clear-Variable CSHTTP_*

    write-host "   ||" -foregroundcolor DarkGray

    #cosmetic closeout of second report segment
        $script:CSToolReport+="==================================`r`n"
        $script:CSToolReport+="`r`n"
        $script:CSToolReport+="= Platform IP Scan results (determined by Platform selection):`r`n"

    #ip array check
        [int]$CSHTTTP_SuccessCount = '0'
        [int]$CSHTTP_ArrayPos = '0'       
        foreach ($iteration in $CSPlatIPs) {

            #configure zeroes, for formatting
            if ($CSHTTP_SuccessCount -lt 10) {
                [string]$CSHTTP_FirstZero = "0"
            } else {
                [string]$CSHTTP_FirstZero = ""
            }

            if ($CSPlatIPs.Count -lt 10) {
                [string]$CSHTTP_SecondZero = "0"
            } else {
                [string]$CSHTTP_SecondZero = ""
            }

            #the IP check itself
            $CSHTTP_ArrayPos++
            $CSHTTP_ServerCheck = (New-Object net.sockets.tcpclient); $CSHTTP_ServerCheck.connect("$iteration",443)
            write-host "`r     Platform IPs               " -nonewline; write-host ('[' + $CSHTTP_FirstZero + $CSHTTP_SuccessCount + '/' + $CSHTTP_SecondZero + $CSPlatIPs.count + ']') -NoNewline -foregroundcolor DarkGray

            #conditional code to write results to the log as they file in
            if ($CSHTTP_ServerCheck.Connected) {
                $script:CSToolReport+= "`+ SUCCESS: $iteration `r`n"
                $CSHTTP_SuccessCount++
                } else {
                $script:CSToolReport+= "`- FAILURE: $iteration `r`n"
                }

            #wait a little bit
            Start-Sleep -m 100

            #configure zeroes again, for formatting
            if ($CSHTTP_SuccessCount -lt 10) {
                [string]$CSHTTP_FirstZero = "0"
            } else {
                [string]$CSHTTP_FirstZero = ""
            }

            #finish up
            if ($CSHTTP_ArrayPos -eq $CSPlatIPs.count) {
                write-host "`r     Platform IPs               " -NoNewline;
                if ($CSHTTP_SuccessCount -ne $CSPlatIPs.count) {
                    $script:CSIPCheckError = $true
                    write-host ('[' + $CSHTTP_FirstZero + $CSHTTP_SuccessCount + '/' + $CSHTTP_SecondZero + $CSPlatIPs.count + ']') -foregroundcolor Red -nonewline
                } else {
                    write-host ('[' + $CSHTTP_FirstZero + $CSHTTP_ArrayPos + '/' + $CSHTTP_SecondZero + $CSPlatIPs.count + ']') -foregroundcolor Green -nonewline
                }
            }
        $CSHTTP_ServerCheck.close()
        }
        Clear-Variable CSHTTP_*

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #cagver
        write-host "Latest Agent Build    " -nonewline
        [xml]$CSHTTP_XML = (New-Object System.Net.WebClient).DownloadString("$CAGVerAddress")
        If ($CSHTTP_XML) { 
            Write-Host "       " $CSHTTP_XML.UpdateStatus.CurrentVersion -ForegroundColor Yellow
            $script:CSAgentVersionOnline = $CSHTTP_XML.UpdateStatus.CurrentVersion
        } else {
            Write-host "Inaccessible" -foregroundcolor Red
        }
        Clear-Variable CSHTTP_*

    #remove proxy settings from defaults and delete variables
        $PSDefaultParameterValues.Remove('Invoke-WebRequest:Proxy')
        $PSDefaultParameterValues.Remove('Invoke-WebRequest:ProxyCredential')

    #output if an IP check failed
    if ($script:CSIPCheckError) {
        write-host "     Individual IP check failures do not necessarily indicate connection issues." -ForegroundColor Green
    }

    write-host "`r"
}

Function SystemInfo {
    #list basic computer info
    write-host "Device Name: $env:Computername - Build $CSKernel3 /" (get-WMiObject -computername $env:computername -Class win32_operatingSystem).caption
    write-host "`r"
}

Function HealthCheck {

#here we are again. what a long, strange journey this has been.

    #write to the log file
        $script:CSToolReport+="==================================`r`n"
        $script:CSToolReport+= "`r`n"
        $script:CSToolReport+="= Endpoint and Agent Health Check Results:`r`n"

    #monitor process check
        write-host "     Monitor Service State     " -NoNewline
        if (@(get-process AEMAgent).count -lt 1) {
            write-host " Offline" -ForegroundColor Red -nonewline
            $script:CSToolReport+= "`- FAILURE: AEMAgent Monitor Service not running.`r`n"
            $CSMonitorRunning = $false
        } else {
            $CSMonitorRunning = $true
            write-host "  Online" -ForegroundColor Green -NoNewline
            $script:CSToolReport+= "`+ SUCCESS: AEMAgent Monitor Service running.`r`n"
        }

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #agent running check
        write-host "Agent Service State       " -NoNewline
        if (@(get-process CagService).count -lt 1) {
            write-host " Offline" -ForegroundColor Red
            $script:CSToolReport+= "`- FAILURE: CagService Agent Service not running `r`n"
            $CSAgentRunning = $false
        } else {
            $CSAgentRunning = $true
            write-host "  Online" -ForegroundColor Green
            $script:CSToolReport+= "`+ SUCCESS: CagService Agent Service running. `r`n"
        }

    #agent version check
        write-host "     Agent Service Version     " -NoNewline
        if ($CSAgentRunning) {
            $CSAgentVer=(([System.Diagnostics.FileVersionInfo]::GetVersionInfo("$CSProgramFiles\CentraStage\Core.dll").FileVersion).split(".")[3])
            if ($CSAgentVersionOnline) {
                if ($CSAgentVersionOnline -eq $CSAgentVer) {
                    write-host "  Latest" -foregroundcolor green -nonewline
                    $script:CSToolReport+= "`+ SUCCESS: Agent is up-to-date.`r`n"
                } else {
                    write-host "Outdated" -ForegroundColor Red -nonewline
                    $script:CSToolReport+= "`- FAILURE: Endpoint is running Agent build $CSAgentVer`; latest is $CSAgentVersionOnline`.`r`n"
                }
            } else {
                write-host " Unknown" -ForegroundColor Red -NoNewline
                $script:CSToolReport+= "`: UNKNOWN: Endpoint is running Agent build $CSAgentVer, but no comparison data was available.`r`n"
            }
        } else {
            write-host " Unknown" -ForegroundColor Red -NoNewline
            $script:CSToolReport+= "`: UNKNOWN: Could not ascertain version of Agent on Endpoint.`r`n"
        }

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #duplicate nlog.dll check
        write-host "NLog.dll Clashes          " -nonewline
        $CSPathTest = @(Get-ChildItem -Path "C:\Windows\Microsoft.NET\assembly" -Recurse | Where-Object {$_.Name -match "^nlog\.dll"})
        if ($CSPathTest.length -gt 0) {
                $CSDupeFound = $true
                $script:CSToolReport+= "`- FAILURE: NLog.dll was found in the Endpoint`'s GAC, which can cause the Agent not to start.`r`n"
                $script:CSToolReport+= "  If the Agent does not start, remove NLog.dll from the C:\Windows\Microsoft.NET\Assembly location.`r`n"
        }
        if (!$CSDupeFound) {
            write-host " 0 Found" -ForegroundColor Green
            $script:CSToolReport+= "`+ SUCCESS: No NLog.dll clashes were found on Endpoint.`r`n"
        } else {
            write-host "Detected" -foregroundcolor Red
        }

    #operating system check
        write-host "     Operating System Version  " -nonewline
        if ($CSKernel3 -eq 3790) {
            write-host " Caution" -foregroundcolor yellow -nonewline
            $script:CSToolReport+= "`+ CAUTION: Datto RMM is no longer guaranteed to work on Windows Server 2003.`r`n"
        } elseif ($CSKernel3 -gt 2600) {
            write-host "     OK!" -foregroundcolor green -nonewline
            $script:CSToolReport+= "`+ SUCCESS: Endpoint is running a supported Operating System.`r`n"
        } else {
            if ($CSKernel4 -lt 6532) {
                write-host "Outdated" -ForegroundColor Red -nonewline
                $script:CSToolReport+= "`- FAILURE: Endpoint is not running a supported Operating System.`r`n"
            } else {
                write-host " Caution" -foregroundcolor yellow -nonewline
                $script:CSToolReport+= "`+ CAUTION: Datto RMM is no longer guaranteed to work on Windows XP.`r`n"
            }
        }

    write-host "   ||   " -foregroundcolor DarkGray -nonewline
    
    #FIPS-compliance check
        $ErrorActionPreference = "stop" #change errorActionPref because registry; re-engaged post-dotnet check
        write-host "FIPS-Compliance           " -nonewline
        try {$CSFIPSComp=((Get-ItemProperty -Path "HKLM:\System\CurrentControlSet\Control\Lsa\FIPSAlgorithmPolicy" -Name Enabled).Enabled)}
        catch [System.Exception] {
                #do nothing, the next block caters for this
        }
        if ($CSFIPSComp -ilike "1") {
            write-host " Engaged" -ForegroundColor Red
            $script:CSToolReport+= "`- FAILURE: Endpoint enforces FIPS-Compliance. This setting is known to hinder Platform connectivity.`r`n"
        } else {
            write-host "Disabled" -foregroundcolor green
            $script:CSToolReport+= "`+ SUCCESS: FIPS-Compliance, which can cause connectivity issues, is not mandated on this device.`r`n"
        }

    #dotnet minimum compat check
        write-host "    .NET Framework Version     " -nonewline
        try {$script:CS403Inst=((Get-ItemProperty -Path "$CSRegSoftware\Microsoft\Updates\Microsoft .NET Framework 4 Extended\KB2600211" -Name ThisVersionInstalled).ThisVersionInstalled)}
        catch [System.Exception] {
            write-host "Outdated" -ForegroundColor Red -nonewline
            $script:CSToolReport+= "`- FAILURE: Endpoint must have at least .NET Framework (Full) v4.0.3 installed for Agent to function.`r`n"
        }
        if ($script:CS403Inst -match "Y") {
            write-host "     OK!" -ForegroundColor Green -NoNewline
            $script:CSToolReport+= "`+ SUCCESS: .NET Framework 4.0.3 is installed (potentially alongside other versions).`r`n"
        } else {
            write-host "Outdated" -ForegroundColor Red -nonewline
            $script:CSToolReport+= "`- FAILURE: Endpoint must have at least .NET Framework (Full) v4.0.3 installed for Agent to function.`r`n"
        }

        #stick a 4.5 notice in the log
        try {
            $CSNet45Check=(Get-ItemProperty -Path "HKLM:\Software\Microsoft\NET Framework Setup\NDP\v4\Full" -Name Release).Release # map to a variable we won't use to keep it quiet
            $script:CSToolReport+= "`+ SUCCESS: Device has version 4.5 of the .NET Framework or higher installed: If device is a Network Node, ESXi monitoring will function.`r`n"
        } 
        catch [System.Exception] {
            $script:CSToolReport+= "`- CAUTION: Device lacks version 4.5 of the .NET Framework. If device is a Network Node, it will be unable to monitor ESXi devices.`r`n"
        }

        $ErrorActionPreference = "silentlycontinue" #as promised

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #splashtop install/version check
        write-host "Splashtop Version         " -nonewline
        try {$CSStopVer=([System.Diagnostics.FileVersionInfo]::GetVersionInfo("C:\Program Files (x86)\Splashtop\Splashtop Remote\Server\srserver.exe").ProductVersion)}
        catch [System.Exception] {write-host "     N/A" -ForegroundColor Yellow}
        if ($CSStopVer) {
            write-host " $CSStopVer" -ForegroundColor Yellow
            $script:CSToolReport+= "`: MENTION: Splashtop Streamer version installed is $CSStopVer`.`r`n"
        }

    #system disk free space
        write-host "     System Drive Free Space   " -nonewline
        if ([math]::Round((get-WmiObject win32_logicaldisk -Filter "DeviceID='$env:SystemDrive'").FreeSpace /1GB) -lt 1) {
            write-host "Need >1G" -ForegroundColor Red -nonewline
            $script:CSToolReport+= "`- FAILURE: Endpoint requires at least 1GB of free space on the System Drive to function properly.`r`n"
        } else {
            write-host ">1G Free" -ForegroundColor Green -NoNewline
            $script:CSToolReport+= "`+ SUCCESS: Endpoint has at least 1GB of free space on the System Drive.`r`n"
        }
    
    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #closeout cosmetique of prior section
        $script:CSToolReport+="==================================`r`n"
        $script:CSToolReport+= "`r`n"

    #cagservice error events (ported from function)
        write-host "CagService Event Log      " -nonewline
        $script:CSToolReport+="= Summary of CagService-related problem events from last 7 days of Event Log:`r`n"
        $CSEventLogData = Get-EventLog -LogName System -Source 'Service Control Manager' -After (Get-Date).AddDays(-7)| Where-Object -FilterScript {($_.Message -match 'CentraStage') -and ($_.EventID -notmatch '^(7036|7045)$')} | Select-Object -Property EventID, Message, TimeWritten | Group-Object -Property Message
        if ($CSKernel3 -gt 9000) {
            #fancy powershell 5 version
            foreach ($Event in $CSEventLogData) {
                if ($event) {
                    $CSLogsPresent=$true
                    $script:CSToolReport += "- FAILURE: Noteworthy CagService Event Log entry discovered:`r`n"
                    $script:CSToolReport += "Event of ID " + $event.Group.eventID + " logged at " + $event.group.TimeWritten + "`r`n"
                    $script:CSToolReport += ": : : : : : : : : : : : : : : : : : :`r`n"
                    $script:CSToolReport += $event.Group.message
                    $script:CSToolReport += "`r`n- - - - - - - - - - - - - - - - - - -`r`n"
                }
            }
        } else {
            #gruel for powershell 2 users - ignore XP because it handles logs differently
            if ($CSKernel3 -ge 7000) {
                if ($CSEventLogData) {
                    $CSLogsPresent=$true
                    $script:CSToolReport += "- FAILURE: Noteworthy CagService Event Log entries noted in attached CSV.`r`n"
                    if (!$CSResultDirectory) {
                        makeFolder
                    }
                    $CSEventLogData | Export-CSV -path "$CSResultDirectory\AHC-$env:computername-EventLog-$(EpochTime).csv"
                }
            }
        }

        if (!$CSLogsPresent) {
            write-host "0 Issues" -ForegroundColor Green
            $script:CSToolReport += "`+ SUCCESS: No noteworthy CagService-related Event Logs were discovered.`r`n"
        } else {
            $script:CSToolReport += "`: No further Event Log entries of note were discovered.`r`n"
            write-host "  Issues" -ForegroundColor Red
        }

    #registry validation (ported from function)
        write-host "     Registry Validation       " -nonewline
        $CSRegLocations = @('hklm:\Software\Microsoft\Windows\Currentversion\Uninstall','hklm:\Software\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall','hkcu:\Software\Microsoft\Windows\CurrentVersion\Uninstall')
        $script:CSToolReport+="==================================`r`n"
        $script:CSToolReport+="`r`n"
        $script:CSToolReport+="= Registry Scan results:`r`n"

        foreach ($arrayPosition in $CSRegLocations) {
            foreach ($iteration in Get-ChildItem "$arrayPosition") {
                $CSregSearch=Get-ItemProperty -Path "registry::$iteration" | Where-Object { $_ -match "[^\u0009\u000a\u000d\u0020-\uD7FF\uE000-\uFFFD]"}
                if ($CSregSearch) {
                    $CSRegObject = $CSregSearch.PSObject.Properties | Where-Object { $_ -match "[^\u0009\u000a\u000d\u0020-\uD7FF\uE000-\uFFFD]"}
                    $CSRegistryIssues = $true
                    $script:CSToolReport += "- FAILURE: Registry Object with invalid Unicode data located.`r`n"
                    $script:CSToolReport += "`>- Registry Key: $iteration`r`n"
                    $script:CSToolReport += "`>--  Value name: " + $CSRegObject.name + "`r`n"
                    $script:CSToolReport += "`>--  Value data: " + $CSRegObject.value + "`r`n"
                    $script:CSToolReport += "- - - - - - - - - - - - - - - - - - -`r`n"
                }
            }
        }

        if ($CSRegistryIssues) {
            write-host "  Issues" -ForegroundColor Red -nonewline
            $script:CSToolReport += "`: No further Registry Objects with invalid Unicode data were discovered.`r`n"
        } else {
            write-host "0 Issues" -ForegroundColor Green -nonewline
            $script:CSToolReport += "`+ SUCCESS: No invalid Unicode data was found in the Registry.`r`n"
        }

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #agent XML checks (ported from function)
        write-host "Agent File Validation     " -nonewline
        $script:CSToolReport+="= Agent File Scan results:`r`n"
        if (!$CSAdmin) {
            write-host "No Admin" -foregroundcolor Red
            $script:CSToolReport+="`: UNKNOWN: Tool was run with insufficient access to analyse Agent files.`r`n"
        } else {
            [array]$CSXMLFiles = [array]$CSXMLFiles += (Get-ChildItem -Path 'C:\Documents and Settings\Default User\Local Settings\Application Data\CentraStage' -Filter 'user.config' -Recurse -ErrorAction SilentlyContinue -Force).fullname
            [array]$CSXMLFiles = [array]$CSXMLFiles += (Get-ChildItem -Path 'C:\Documents and Settings\LocalService\Local Settings\Application Data\CentraStage' -Filter 'user.config' -Recurse -ErrorAction SilentlyContinue -Force).fullname
            [array]$CSXMLFiles = [array]$CSXMLFiles += (Get-ChildItem -Path 'C:\Windows\System32\config\systemprofile\AppData\Local\CentraStage' -Filter 'user.config' -Recurse -ErrorAction SilentlyContinue -Force).fullname
            [array]$CSXMLFiles = [array]$CSXMLFiles += (Get-ChildItem -Path 'C:\Windows\SysWOW64\config\systemprofile\AppData\Local\CentraStage' -Filter 'user.config' -Recurse -ErrorAction SilentlyContinue -Force).fullname

            foreach ($CSXMLFile in $CSXMLFiles) {
                if ($CSXMLFile) {
                    $CSXMLTest = New-Object System.Xml.XmlDocument
                    try {
                        $CSXMLTest.Load("$CSXMLFile")
                    } catch [System.Xml.XmlException] {
                        $script:CSXMLError=$true
                        $script:CSToolReport += "- FAILURE: $CSXMLFile is corrupt.`r`n"
                    }
                }
            }

            # check submittedpatches.json
            try {
                $CSPatchJSON = get-content "C:\ProgramData\CentraStage\SubmittedPatches.json" -ErrorAction Stop
            } catch {
                #no file, no problem
            }

            if ($CSPatchJSON) {
		        if ($CSPatchJSON.Substring(0,2) -notmatch "^\[\{$" -or $CSPatchJSON.Substring($CSPatchJSON.Length-2) -notmatch "^\}\]$") {
                    $script:CSToolReport += "- FAILURE: C:\ProgramData\CentraStage\SubmittedPatches.json is corrupt.`r`n"
		            $script:CSToolReport += "- Please delete this file or re-install the Agent.`r`n"
                    $script:CSXMLError=$true
                }
            }

            #display results
            if ($script:CSXMLError) {
                write-host "  Issues" -ForegroundColor Red
            } else {
                write-host "0 Issues" -foregroundcolor Green
                $script:CSToolReport += "+ SUCCESS: Agent XML files verified.`r`n"
            }

        } # admin check

    #compare system time to google's HTTP headers -- thanks to chris H for the suggestion
        $script:CSToolReport += "= Local/Online time readout comparison:`r`n"
        
        #get google's date from its http headers
        $varServerTime=([DateTimeOffset]::Parse((Get-HTTPHeaders http://www.google.com | Where-Object {$_.Key -eq "Date"}).Value)).UtcDateTime
        $varServerEpoch=[int64](($varServerTime)-(get-date "1/1/1970")).TotalSeconds

        #get local time
        $varLocalEpoch=[int64](([datetime]::UtcNow)-(get-date "1/1/1970")).TotalSeconds

        #compare'm'tgnph
        [int]$varTimeOffset = $varServerEpoch - $varLocalEpoch
        if ($varTimeOffset -lt 0) {$varTimeOffset=0-$varTimeOffset}

        #get to printing
        write-host "     Clock Accuracy Offset  " -NoNewline

        if (($varTimeOffset).tostring().length -ge 4) {
            write-host ">15 Min Out" -NoNewline -foregroundcolor Red
            $script:CSToolReport += "- FAILURE: Device time is incorrect by more than fifteen minutes.`r`n"
            $script:CSToolReport += "  This will render the device unable to communicate via SSL, precluding the Agent from connecting.`r`n"
        } elseif ($varTimeOffset -ge 600) {
            write-host ">10 Min Out" -NoNewline -foregroundcolor Red
            $script:CSToolReport += "- FAILURE: Device time is incorrect by more than ten minutes.`r`n"
            $script:CSToolReport += "  This will render the device unable to communicate via SSL, precluding the Agent from connecting.`r`n"
        } elseif ($varTimeOffset -ge 121) {
            write-host " >2 Min Out" -NoNewline -foregroundcolor Yellow
            $script:CSToolReport += "- CAUTION: Device time is incorrect by two to ten minutes.`r`n"
            $script:CSToolReport += "  A device with inaccurate time may be unable to communicate via SSL, precluding the Agent from connecting.`r`n"
        } elseif ($varTimeOffset -le 120) {
            write-host "   Accurate" -NoNewline -foregroundcolor Green
            $script:CSToolReport += "+ SUCCESS: Device time is accurate within two minutes.`r`n"
            $script:CSToolReport += "  The device and the Agent should be able to communicate via secure channels.`r`n"
        }

    write-host "   ||   " -foregroundcolor DarkGray -nonewline

    #uptime
        $CSLastBootTime=[Management.ManagementDateTimeConverter]::ToDateTime((Get-WmiObject Win32_OperatingSystem).LastBootupTime) | New-TimeSpan
        write-host "System Uptime         " -NoNewline

        if (($CSLastBootTime.Days -as [String]).Length -eq 3) {
            $CSBootString='{0}D ' -f $CSLastBootTime.Days
        } elseif (($CSLastBootTime.Days -as [String]).Length -eq 2) {
            $CSBootString=' {0}D ' -f $CSLastBootTime.Days
        } else {
            $CSBootString=' 0{0}D ' -f $CSLastBootTime.Days
        }

        if (($CSLastBootTime.Hours -as [String]).Length -eq 1) {
            $CSBootString+='0{0}H ' -f $CSLastBootTime.Hours
        } else {
            $CSBootString+='{0}H ' -f $CSLastBootTime.Hours
        }

        if (($CSLastBootTime.Minutes -as [String]).Length -eq 1) {
            $CSBootString+='0{0}M ' -f $CSLastBootTime.Minutes
        } else {
            $CSBootString+='{0}M ' -f $CSLastBootTime.Minutes
        }

        write-host $CSBootString -ForegroundColor Yellow
        $script:CSToolReport += "= System Uptime: $CSBootString`r`n"
            
    #close function
}

Function NETRepair {
    write-host "- Analyse and repair Microsoft .NET Framework installation?: " -NoNewline
    $choiceInput = read-host
    switch -Regex ($choiceInput) {
   
        'A|a|Y|y' {
            if (test-path "$CSScriptDir\NetFxRepairTool.exe") {
                write-host "-- Locally running / collecting logs from .NETRepair. Please wait..."
            } else {
                write-host "-- Downloading / running / collecting logs from .NETRepair. Please wait..."
                (New-Object System.Net.WebClient).DownloadFile("https://download.microsoft.com/download/2/B/D/2BDE5459-2225-48B8-830C-AE19CAF038F1/NetFxRepairTool.exe", "$CSScriptDir\NetFXRepairTool.exe")
            }

            if (!$CSResultDirectory) {
                makeFolder
            }

            #cmd /c-with-spaces code that works on powershell 2 and 5 -- why did MS break this?
            new-item -path run.bat -value "`"$CSScriptDir\NetFxRepairTool.exe`" /q /n /l `"$CSResultDirectory`"" -itemtype file -force | out-null
            cmd /c run.bat | out-null
            remove-item run.bat | out-null

            start-sleep 5
            $CSCabFile=(gci | sort LastWriteTime | select -last 1)
            if (Test-Path "$CSScriptDir\$CSCabFile" -NewerThan (Get-Date).AddSeconds(-15)) {
                move-item $CSCabFile -Destination "$CSResultDirectory\AHC-$env:computername-NETRepair-$(EpochTime).cab"
            }
            $script:CSToolReport+= "`: MENTION: Microsoft .NET Repair tool was run on the Endpoint.`r`n"
        }

        default {
            # don't not be doing not anything
        }
    }
}

function sevenZip {
    # zip is no longer an option
    # check to see if the 7-zip command-line executable is available:

    # powershell 2: give CSZipApp a deliberately fake value so the comparison works (seriously???)
    $script:CSZipApp = "$CSScriptDir\brokenpath.sgl"

    # - registry: 32-bit and 64-bit natives
    if (test-path "HKLM:\SOFTWARE\7-zip") {
        #found -- set "Path" to $script:CSZipApp variable
        $script:CSZipApp = (Get-ItemProperty -Path "HKLM:\SOFTWARE\7-zip" -Name Path).Path + "7z.exe"
    } 
    
    # - registry: 32-bit on 64-bit windows
    if (test-path "HKLM:\SOFTWARE\WOW6432Node\7-zip") {
        #found -- set "Path" (WOW) to $script:CSZipApp variable
        $script:CSZipApp = (Get-ItemProperty -Path "HKLM:\SOFTWARE\WOW6332Node\7-zip" -Name Path).Path + "7z.exe"
    } 
    
    # local override
    if (test-path "$CSScriptDir\7za.exe") {
        $script:CSZipApp = $CSScriptDir + "\7za.exe"
    }

    # test to make sure the logic has actually found 7z/7za.exe
    if (!(test-path "$script:CSZipApp")) {
        #not found -- download and then set to $script:CSZipApp variable
        write-host "- Downloading 7-zip portable to Tool directory..."
        (New-Object System.Net.WebClient).DownloadFile("https://storage.centrastage.net/7zip.zip", "$CSScriptDir\7zip.zip")

        # i hate this code: UNZIPPER for ps2+
        $CSZipPath = "$CSScriptDir\7zip.zip"
        $CSZipFile = (new-object -com shell.application).NameSpace($CSZipPath)
        $CSZipDest1 = "$CSScriptDir"
        $CSZipDest2 = (new-object -com shell.application).NameSpace($CSZipDest1)
        $CSZipDest2.CopyHere($CSZipFile.Items())
        $script:CSZipApp = $CSScriptDir + "\7za.exe"
    }

    if (!(test-path "$script:CSZipApp")) {
        # if we still don't have anything, throw toys from pram
        write-host "7-zip could not be obtained; log files cannot be catalogued."
        write-host "Please report this error!"
        write-host "Exiting..."
        cmd /c pause
        exit
    }
}

fUnCtIoN GrabLogs {

    if (!$CSResultDirectory) {
        makeFolder
    }

    new-item "$CSResultDirectory\AEMLogs\" -type directory -Force | out-null
    new-item "$CSResultDirectory\AEMLogs\Config" -type directory -Force | out-null
    new-item "$CSResultDirectory\AEMLogs\Config64" -type directory -Force | out-null
    new-item "$CSResultDirectory\AEMLogs\AppData" -type directory -Force | out-null
    new-item "$CSResultDirectory\AEMLogs\ProgFiles" -type directory -Force | out-null

    # config 1
    if ($CSKernel3 -lt 5000) {
        if ((Get-WmiObject -Class win32_ComputerSystem -namespace "root\CIMV2").DomainRole -le 1) {
            #workstation
            if (([IntPtr]::size) -eq 4) {
                #32-bit
                Copy-Item -Path "C:\Documents and Settings\LocalService\Local Settings\Application Data\CentraStage" -Recurse -Destination "$CSResultDirectory\AEMLogs\Config"
            } else {
                #64-bit
                Copy-Item -Path "C:\Documents and Settings\Default User\Local Settings\Application Data\CentraStage" -Recurse -Destination "$CSResultDirectory\AEMLogs\Config"
            }
        } else {
            #server
            Copy-Item -Path "C:\Documents and Settings\Default User\Local Settings\Application Data\CentraStage" -Recurse -Destination "$CSResultDirectory\AEMLogs\Config"
        }
    } else {
        Copy-Item -Path "C:\Windows\System32\config\systemprofile\AppData\Local\CentraStage" -Recurse -Destination "$CSResultDirectory\AEMLogs\Config"
        if (([IntPtr]::size) -eq 8) {Copy-Item -Path "C:\Windows\SysWOW64\config\systemprofile\AppData\Local\CentraStage" -Recurse -Destination "$CSResultDirectory\AEMLogs\Config64"}
    }

    # config 2
    cd $env:ALLUSERSPROFILE
    cd "Application Data" -ErrorAction SilentlyContinue
    cd "CentraStage"
    if ($?) {
        Get-ChildItem | Where-Object {$_.name -notmatch "Packages"} | Copy-Item -Recurse -Destination "$CSResultDirectory\AEMLogs\AppData" -Container -ErrorAction SilentlyContinue
    }
    cd $CSScriptDir

    # program files
    Get-ChildItem -Path "$CSProgramFiles\Centrastage" -Recurse | Where-Object { $_.Name -match "^log." } | Copy-Item -Destination "$CSResultDirectory\AEMLogs\ProgFiles"

    # zip it all up (ps2-ps5 compat) -- removed \aemlogs so we don't leave the event log data
    $script:CSLogArchive="$CSScriptDir\AHC-$env:ComputerName-AEMAgentLogs-$(EpochTime).zip"
    new-item -path run.bat -value "`"$script:CSZipApp`" a `"$script:CSLogArchive`" `"$CSResultDirectory`"" -itemtype file -force | out-null
    cmd /c run.bat | out-null

    # clean up
    remove-item run.bat -force | out-null
    remove-item "$CSResultDirectory" -Recurse -Force | out-null
    remove-item 7zip.zip -force | out-null
}

fUnCtIoN GrabLogsBlue {

    if (!$CSResultDirectory) {
        makeFolder
    }

    new-item "$CSResultDirectory\AgentLogs\" -type directory -Force | out-null
    new-item "$CSResultDirectory\AgentLogs\Config" -type directory -Force | out-null
    new-item "$CSResultDirectory\AgentLogs\Config64" -type directory -Force | out-null
    new-item "$CSResultDirectory\AgentLogs\AppData" -type directory -Force | out-null
    new-item "$CSResultDirectory\AgentLogs\ProgFiles" -type directory -Force | out-null

    # config 1
    if ($CSKernel3 -lt 5000) {
        if ((Get-WmiObject -Class win32_ComputerSystem -namespace "root\CIMV2").DomainRole -le 1) {
            #workstation
            if (([IntPtr]::size) -eq 4) {
                #32-bit
                Copy-Item -Path "C:\Documents and Settings\LocalService\Local Settings\Application Data\CentraStage" -Recurse -Destination "$CSResultDirectory\AgentLogs\Config"
            } else {
                #64-bit
                Copy-Item -Path "C:\Documents and Settings\Default User\Local Settings\Application Data\CentraStage" -Recurse -Destination "$CSResultDirectory\AgentLogs\Config"
            }
        } else {
            #server
            Copy-Item -Path "C:\Documents and Settings\Default User\Local Settings\Application Data\CentraStage" -Recurse -Destination "$CSResultDirectory\AgentLogs\Config"
        }
    } else {
        Copy-Item -Path "C:\Windows\System32\config\systemprofile\AppData\Local\CentraStage" -Recurse -Destination "$CSResultDirectory\AgentLogs\Config"
        if (([IntPtr]::size) -eq 8) {Copy-Item -Path "C:\Windows\SysWOW64\config\systemprofile\AppData\Local\CentraStage" -Recurse -Destination "$CSResultDirectory\AgentLogs\Config64"}
    }

    # config 2
    cd $env:ALLUSERSPROFILE
    cd "Application Data" -ErrorAction SilentlyContinue
    cd "CentraStage"
    Get-ChildItem | Where-Object {$_.name -notmatch "Packages"} | Copy-Item -Recurse -Destination "$CSResultDirectory\AgentLogs\AppData" -Container -ErrorAction SilentlyContinue
    cd $CSScriptDir

    # program files
    Get-ChildItem -Path "$CSProgramFiles\Centrastage" -Recurse | Where-Object { $_.Name -match "^log." } | Copy-Item -Destination "$CSResultDirectory\AgentLogs\ProgFiles"

}

fuNctIon WMIRepair {
    write-host "`- Analyse the WMI for errors? (This can take upwards of 15 minutes!): " -NoNewline
    $choiceInput = read-host
    switch -Regex ($choiceInput) {
   
        'A|a|Y|y' {
            if (test-path "$CSScriptDir\WMIDiag.vbs") {
                write-host "-- Locally running / collecting logs from WMIDiag. Please wait..."
            } else {
                #download wmidiag-dot-vbs
                write-host "-- Downloading / running / collecting logs from WMIDiag. Please wait..."
                (New-Object System.Net.WebClient).DownloadFile("https://storage.centrastage.net/WMIDiag.vbs", "$CSScriptDir\WMIDiag.vbs")
            }
            #run it
            cscript /nologo `"$CSScriptDir\WMIDiag.vbs`" | out-null
            #move the log files
            if (!$CSResultDirectory) {
                makeFolder
            }
            Move-Item "$env:temp\WMIDIAG-V2.2_*" "$CSResultDirectory"
            $script:CSToolReport+= "`: MENTION: Microsoft WMI Diagnostic Tool was run on the Endpoint.`r`n"
        }

        default {
            # do less than something
        }
    }
}

################################# MAIN SCRIPT BODY #########################################
##################################### PAGE ONE #############################################

write-host "Page 1: Platform Connectivity" -ForegroundColor Green
write-host "Started @ $(Get-Date)" -ForegroundColor Cyan
write-Host "============================="

#perform an agent version check
#blue: uncomment this next line
#<#
[xml]$CSToolVer_XML = (New-Object System.Net.WebClient).DownloadString("https://storage.centrastage.net/version.xml")

if (!$CSToolVer_XML) {
    write-host 'Could not check tool version. Please whitelist https://storage.centrastage.net.' -ForegroundColor Red
    write-host `r
} else {
    if ($CSToolVer_XML.seagull.build -notmatch $CStitle) {
        write-host "Agent Health Check Tool is out of date." -ForegroundColor Red
        write-host 'Download the latest build from: ' -NoNewline
        write-host 'http://dat.to/ahcdl' -ForegroundColor Cyan
        cmd /c pause
        write-host `r
    }
}
#blue: uncomment this next line
##>

#prompt to restart as administrator if not currently
if (!$CSAdmin) {
    write-host "Script is not being run as an Administrator." -ForegroundColor Red
    write-host "Re-launch as Admin to enable gathering/analysis of Agent operational data?"
    write-host "[ENTER]" -NoNewline -foregroundcolor Red; write-host "/No, " -NoNewline; write-host "[A]" -NoNewline -foregroundcolor Red; write-host "/Yes: " -NoNewline
    $choiceInput = read-host
    switch -Regex ($choiceInput) {
        default {
        #do nothing
        }
        'A|a|Y|y' {
            #write our current directory to a hard file since admin launches in sys32
            $pwd.path | out-file "$env:TEMP\~AEMHC.tmp"
            #relaunch the shell in admin-mode
            $newProcess = new-object System.Diagnostics.ProcessStartInfo "PowerShell";
            $newProcess.Arguments = "-executionpolicy bypass &'" + $script:MyInvocation.MyCommand.Path + "'"
            $newProcess.Verb = "runas";
            [System.Diagnostics.Process]::Start($newProcess);
            exit
        }
}
    write-host "`r"
} else {
    #congratulate the user
    write-host "Administrative permissions confirmed." -ForegroundColor Cyan
    write-host "`r"
    #search for the .tmp file, if it was found, load and delete it
    if (!(test-path "$env:TEMP\~AEMHC.tmp")) {
        #do nothing, user launched powershell first and then the script
    } else {
        get-content "$env:TEMP\~AEMHC.tmp" | cd
        Remove-Item "$env:TEMP\~AEMHC.tmp"
        $CSScriptDir = $pwd.path
    }
}

#write basic information to the IP report
$script:CSToolReport= "$CStitle`: Verbose Output Log`r`n"
$script:CSToolReport+= "==================================`r`n"
$script:CSToolReport+= "`: Endpoint Hostname:                $env:computername`r`n"
$script:CSToolReport+= "`: Endpoint Windows Version:         Build $CSKernel3 / "
$script:CSToolReport+= (get-WMiObject -computername $env:computername -Class win32_operatingSystem).caption + "`r`n"
$script:CSToolReport+= "`: Date/Time of Tool Execution:      $(Get-Date)`r`n"

#auto-detect the server of the installed agent
[xml]$CSAutoDetect_XML = get-content "$CSProgramFiles\CentraStage\CagService.exe.config" -EA SilentlyContinue
[string]$CSAutoDetect_STRING = ($CSAutoDetect_XML.configuration.applicationSettings."CentraStage.Cag.Core.AppSettings".setting | Where-Object {$_.Name -eq 'CsIp'}).value
    if ($CSAutoDetect_STRING -match '01cc') {$CSAutoDetect = "Pinotage"}
    if ($CSAutoDetect_STRING -match '02cc') {$CSAutoDetect = "Merlot"}
    if ($CSAutoDetect_STRING -match '03cc') {$CSAutoDetect = "Zinfandel"}
    if ($CSAutoDetect_STRING -match 'syrahcc') {$CSAutoDetect = "Syrah"}
    if ($CSAutoDetect_STRING -match '01concordcc') {$CSAutoDetect = "Concord"}
    if ($CSAutoDetect_STRING -match '01pandacc') {$CSAutoDetect = "Panda"}
    if ($CSAutoDetect_STRING -match 'deve') {$CSAutoDetect = "RMM Development - do not auto-detect"}
    if ($CSAutoDetect_STRING -match 'stagi') {$CSAutoDetect = "RMM Staging - do not auto-detect"}
    if ($CSAutoDetect_STRING -match 'test') {$CSAutoDetect = "RMM Testing - do not auto-detect"}
    if ($CSAutoDetect_STRING -match 'main') {$CSAutoDetect = "RMM Maintenance - do not auto-detect"}

write-host "                               -== Connectivity Check ==-" -foregroundcolor Green

#blue: uncomment the next line
#<#
#pick a server or configure a proxy
if ($CSAutoDetect) {
    write-host "                  0" -ForegroundColor Yellow -NoNewline; write-host " Infer Platform selection from Agent (" -ForegroundColor White -NoNewline;`
    write-host "$CSAutoDetect" -ForegroundColor Cyan -NoNewline; write-host ")" -ForegroundColor White
}

write-host "      1" -ForegroundColor Yellow -NoNewline; write-host " Pinotage (EMEA West 1) " -ForegroundColor White -NoNewline;`
write-host "|" -ForegroundColor DarkGray -NoNewline; write-host " 2" -ForegroundColor Yellow -NoNewline; write-host " Merlot (EMEA West 2) " -ForegroundColor White -NoNewline;`
write-host "|" -ForegroundColor DarkGray -NoNewline; write-host " 3" -ForegroundColor Yellow -NoNewline; write-host " Syrah (Asia-Pacific) " -ForegroundColor White
write-host "         4" -ForegroundColor Yellow -NoNewline; write-host " Zinfandel (US West) " -ForegroundColor White -NoNewLine;`
write-host "|" -ForegroundColor DarkGray -NoNewline; write-host "  5" -ForegroundColor Yellow -NoNewline; write-host " Concord (US East)   " -ForegroundColor White -NoNewLine;`
write-host "|" -ForegroundColor DarkGray -NoNewline; write-host " 6" -ForegroundColor Yellow -NoNewline; write-host " Configure Proxy" -ForegroundColor White;
#blue: uncomment the next line
##>

#blue: ChoosePlatformBlue instead of ChoosePlatform
while (!$script:CSPlatform) {
    if ($script:CheckAgain) {
        write-host "Please select a number between 1 and 6." -ForegroundColor Red
    }
    ChoosePlatform
}

write-host "Platform Selected: " -ForegroundColor DarkGray -NoNewline; write-host "$script:CSPlatform" -ForegroundColor White
write-host `r
SiteCheck

write-host "Agent Connectivity checks complete." -ForegroundColor Cyan
write-host "Press " -NoNewline; write-host "[ENTER]" -nonewline -ForegroundColor Red; write-host " to advance to Page 2, where Endpoint checks are conducted." -NoNewline
$choiceInput = read-host
switch ($choiceInput) {
    default {
        clear-host
    }
}

##################################### PAGE TWO #############################################

write-host "Page 2: Endpoint Health"
if ($CSAdmin) {
    write-host "Tool is being run as an Administrator    " -ForegroundColor Cyan
} else {
    write-host "Tool is not being run as an Administrator" -ForegroundColor Red
}
write-Host "============================="
write-host "                               -== Endpoint Checks ==-" -foregroundcolor Green
write-host `r
write-host "     " -nonewline
systemInfo
healthCheck

# there was some fancy RTF code here. it's gone now.

write-host "     ============================================================================" -foregroundcolor DarkGray
write-host "     Press for NO               [" -nonewline
write-host "ENTER" -foregroundcolor red -nonewline
write-host "]   " -nonewline
write-host "||" -foregroundcolor DarkGray -nonewline
write-host "   Press for YES                  [" -nonewline
write-host "A" -nonewline -foregroundcolor Red
write-host "]"
write-host `r

$script:CSToolReport+= "`r`n"

if ($script:CS403Inst -match "Y") {
    NETRepair
} else {
    write-host "- Cannot repair .NET Framework (No supported versions found)" -ForegroundColor Red
}

WMIRepair

write-host `r

write-host "Agent Health and Endpoint Checks completed." -ForegroundColor Cyan
write-host "Press [" -NoNewline; write-host "ENTER" -nonewline -ForegroundColor Red; write-host "] to collect the tool's logging data." -NoNewline
$choiceInput = read-host
switch ($choiceInput) {
    default {
        clear-host
    }
}

#################################### PAGE THREE ############################################

write-host "Page 3: Data"
if ($CSAdmin) {
    write-host "Tool is being run as an Administrator    " -ForegroundColor Cyan
} else {
    write-host "Tool is not being run as an Administrator" -ForegroundColor Red
}
write-Host "============================="
write-host "                               -== Data Collection ==-" -foregroundcolor Green
write-host "`r"

#conclude the log
$script:CSToolReport+= "`: Tool finished actions at $(Get-Date).`r`n"

if (!$CSAdmin) {
    write-host "- Copy all Agent log files to Health Check Tool results directory?"
    write-host "  (Tool cannot gather Operational files without Admin permissions.)" -ForegroundColor yellow
} else {
    write-host "- Copy all Agent logging and Operational files to Health Check Tool results directory?"
}
write-host "- This will download a portable version of 7-zip if it is not installed -" -ForegroundColor Yellow

write-host "[ENTER]" -NoNewline -foregroundcolor Red; write-host "/No, " -NoNewline; write-host "[A]" -NoNewline -foregroundcolor Red; write-host "/Yes: " -NoNewline
$choiceInput = read-host
    switch -Regex ($choiceInput) {
   
        'A|a|Y|y' {
            sevenZip
            #blue: GrabLogsBlue
            GrabLogs
        }

        default {
            #do nothing
        }
}

write-host `r

write-host "- Save Agent Health Check scan results to a text file?" -NoNewline
if ($script:CSToolReport -match 'FAIL') {
    write-host ' [1+ failures encountered]' -ForegroundColor Red
} else {
    write-host ' [No failures encountered]' -ForegroundColor Green
}

write-host "[ENTER]" -NoNewline -foregroundcolor Red; write-host "/No, " -NoNewline; write-host "[A]" -NoNewline -foregroundcolor Red; write-host "/Yes: " -NoNewline

$choiceInput = read-host
    switch -Regex ($choiceInput) {
   
        'A|a|Y|y' {
            $CSReportName="AHC-$env:computername-Report-$(EpochTime).log"
            $script:CSToolReport | out-file $CSReportName
            if ($script:CSZipApp) {
	            #this is only defined if the user chooses to zip up logfiles
	            new-item -path run.bat -value "`"$script:CSZipApp`" a `"$script:CSLogArchive`" `"$CSReportName`"" -itemtype file -force | out-null
	            cmd /c run.bat | out-null
	            start-sleep -seconds 5
	            remove-item run.bat -force | out-null
                remove-item $CSReportName -Force | Out-Null
                write-host "- Report file has been created and added to the existing logfile archive."
            } else {
                write-host "- Report file has been created."
            }
        }

        default {
            #do nothing
        }
}

write-host "`r"
write-host "                               -== Scan Completed! ==-" -foregroundcolor Green
write-host `r
write-host "Thank you for using the Agent Health Check Tool. We hope you found it useful."
write-host "Any data collected by the tool can be found in the same directory as the tool itself."
write-host `r
write-host "If you are submitting data for the Support team, please preserve it in its ZIP archive"
write-host "and upload it and any related data to " -nonewline
write-host "http://dat.to/ahcul." -ForegroundColor Cyan
write-host "Only the Support team will be able to access uploaded data." -foregroundcolor Green
#blue: uncomment this next line
##>
write-host `r
write-host "Press [" -nonewline; write-host "ENTER" -NoNewline -foregroundcolor Red
write-host "] to exit, or [" -NoNewline
write-host "A" -foregroundcolor red -nonewline
write-host "] to upload log files: " -nonewline
$choiceInput = read-host
    switch -Regex ($choiceInput) {
        default {
            exit
        }

        'A|a|Y|y' {
            start 'http://dat.to/ahcul'
            write-host "Log Upload website opened. Closing Tool..."
            start-sleep 5
            exit
        }
}